/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.bb;

import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.component.TextField;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.component.ObjectListField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.i18n.ResourceBundle;
import net.rim.device.api.system.Characters;
import net.rim.device.api.ui.container.PopupScreen;
import net.rim.device.api.ui.container.DialogFieldManager;
import javax.microedition.io.file.*;
import javax.microedition.io.Connector;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Stack;
import java.io.*;
import javax.microedition.io.file.FileSystemRegistry;

import com.compactbyte.bibleplus.reader.Util;
import com.compactbyte.bibleplus.reader.MyStringTokenizer;
import net.rim.device.api.system.KeypadListener;

final class AddBibleScreen extends MainScreen implements BiblePlusAppResource
{
	LabelField title ;
	ObjectListField fileList;
	Stack paths = new Stack();
	boolean bibleLoaded = false;

	boolean isBibleLoaded() {
		return bibleLoaded;
	}

	private final static String rootDir =  "file:///";

	private String currentPath = rootDir;

	private String[] rootList;

	private static ResourceBundle _resources = ResourceBundle.getBundle(BUNDLE_ID, BUNDLE_NAME);

	private MenuItem _selectItem = new MenuItem(_resources, SELECT_BIBLE_ACTION, 20, 10) {
			public void run() {
				selectAction();
			}
		};

	private MenuItem _rootDirItem = new MenuItem(_resources, ROOT_DIR, 20, 20) {
			public void run() {
				rootDirAction();
			}
		};



	String getSelectionState() {
		StringBuffer sb = new StringBuffer();
		sb.append(Util.escape(currentPath, '%', ":"));
		for (int i = 0 ; i < paths.size(); i++) {
			String p = (String)paths.elementAt(i);
			sb.append(":");
			sb.append(Util.escape(p, '%', ":"));
		}
		ModelSpecific.log("Sel state: " + sb.toString());
		return sb.toString();
	}

	void restoreSelectionState(String s) {
		if (s==null || s.trim().length()==0) {
			return;
		}
		ModelSpecific.log("Restoring selection state: " + s);

		MyStringTokenizer ms = new MyStringTokenizer(s, ":");
		if (ms.countTokens()==0)
			return;
		currentPath = Util.unescape(ms.nextToken(), '%');

		ModelSpecific.log("current path " + currentPath);

		try {
			FileConnection fileConnection = (FileConnection)Connector.open(currentPath);
			boolean isdir = fileConnection.isDirectory();
			fileConnection.close();
			if (!isdir) {
				currentPath = rootDir;
				return;
			}			
		} catch (IOException e) {
			currentPath = rootDir;
			ModelSpecific.log("IO exception");
		} catch (IllegalArgumentException e2) {
			ModelSpecific.log("illegal argument");
			currentPath = rootDir;
			return;
		} 

		paths.removeAllElements();
		while (ms.hasMoreTokens()) {
			paths.addElement(Util.unescape(ms.nextToken(), '%'));
		}
	}

	/**
	 * Overrides default.  Enter key will take action on directory/file.
	 * Escape key will go up one directory or close application if at top level.
	 *
	 * @see net.rim.device.api.ui.Screen#keyChar(char,int,int)
	 *
	 */
	public boolean keyChar(char c, int status, int time)    {
		switch (c)
		{
		case Characters.ENTER:
			return selectAction();

		case Characters.ESCAPE:
			if ( goBack() )
			{
				return true;
			}

		default:
			return super.keyChar(c, status, time);
		}
	}

	String pathToLoad;

	void closeScreenLater() {
		UiApplication.getUiApplication().invokeLater(new Runnable() {
				public void run() {
					UiApplication uiapp = UiApplication.getUiApplication();
					uiapp.popScreen(uiapp.getActiveScreen());
				}
			});
	}

	void informFailedLater() {
		UiApplication.getUiApplication().invokeLater(new Runnable() {
				public void run() {
					Dialog.inform(_resources.getString(FAILED_LOADING_BIBLE));					
				}
			});
	}

	private void loadBibleFileInDifferentThread() {

		new Thread(new Runnable(){
				public void run() {
					waitdialog.show();

					if (model.loadBible(pathToLoad)) {
						ModelSpecific.log("loaded");
						bibleLoaded = true;
						model.setBibleSelectionDialogState(getSelectionState());
						model.loadChapter();
						waitdialog.close();
						BibleFile b = new BibleFile(model.getVersionName(), pathToLoad);
						model.addBible(b);
						closeScreenLater();
					} else {
						ModelSpecific.log("not loaded");
						waitdialog.close();
						informFailedLater();
					}
				}
			}).start();

	}

	private synchronized void updateList() {
		FileConnection fileConnection = null;
		try {
			fileConnection = (FileConnection)Connector.open(currentPath);
			if (fileConnection.isDirectory()) {
				Enumeration directoryEnumerator = fileConnection.list();
				Vector contentVector = new Vector();
				while(directoryEnumerator.hasMoreElements()) {
					contentVector.addElement(directoryEnumerator.nextElement());
				}
				String[] directoryContents = new String[contentVector.size()];
				contentVector.copyInto(directoryContents);
				fileList.set(directoryContents);
				fileConnection.close();
			} else {
				fileConnection.close();
				//file, make sure extension is PDB
				if (currentPath.toLowerCase().endsWith(".pdb")) {
					pathToLoad = currentPath;		
					currentPath = (String)paths.pop();
					loadBibleFileInDifferentThread();
					
					// if (model.loadBible(currentPath)) {
					// 	bibleLoaded = true;
					// 	currentPath = (String)paths.pop();
					// 	model.setBibleSelectionDialogState(getSelectionState());
					// 	UiApplication uiapp = UiApplication.getUiApplication();
					// 	uiapp.popScreen(uiapp.getActiveScreen());
					// 	ModelSpecific.log("Closing selection");
					// } else {
					// 	Dialog.inform("Failed loading Bible file");
					// 	Dialog.inform(_resources.getString(FAILED_LOADING_BIBLE));
					// }
				} else {
					Dialog.inform(_resources.getString(NOT_PDB_FILE));
					currentPath = (String)paths.pop();
				}
				

			}
		} catch (IOException ex) {
			if (fileConnection!=null) {
				try { 
					fileConnection.close();
				} catch (IOException e) {
				}
			}
		}
	}

	private synchronized void rootDirAction() {
		currentPath = rootDir;
		paths.removeAllElements();
		fileList.set(rootList);
		updateList();		
	}


	private synchronized boolean goBack() {
		if (paths.empty())
			return false;
		currentPath = (String)paths.pop();
		if (currentPath==rootDir) {
			fileList.set(rootList);
		}
		updateList();
		return true;
	}

	private synchronized boolean selectAction() {
		if (fileList.getSelectedIndex()<0)
			return false;
		paths.push(currentPath);
		currentPath += fileList.get(fileList, fileList.getSelectedIndex());
		updateList();
		return true;
	}

	BibleModel model;
	WaitMessageDialog waitdialog;

	public AddBibleScreen(BibleModel _model)   {

		waitdialog = new WaitMessageDialog(_resources.getString(LOADING_BIBLE));

		model = _model;
		title = new LabelField(_resources.getString(APPLICATION_TITLE) , LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
		setTitle(title);
		fileList = new ObjectListField();
		Enumeration rootEnum = FileSystemRegistry.listRoots();
		Vector contentVector = new Vector();
		while (rootEnum.hasMoreElements())       {
			String root = (String)rootEnum.nextElement();
			contentVector.addElement(root);
		}

		rootList = new String[contentVector.size()];
		contentVector.copyInto(rootList);


		String state = model.getBibleSelectionDialogState();
		ModelSpecific.log("state: " + state);

		restoreSelectionState(state);

		if (currentPath==rootDir) {
			fileList.set(rootList);
		} else {
			ModelSpecific.log("calling updatelist");
			updateList();
		}

		add(fileList);

	}

	protected void makeMenu(Menu menu, int instance) {
		super.makeMenu(menu, instance);
		menu.add(_selectItem);
		menu.add(_rootDirItem);
	}


	public void close()   {
		ModelSpecific.log("closing");
		model.setBibleSelectionDialogState(getSelectionState());
		super.close();
	}

	/**
	 * Overrides default implementation.  Performs the select action if the
	 * trackwheel was clicked; otherwise, the default action occurs.
	 *
	 * @see net.rim.device.api.ui.Screen#navigationClick(int,int)
	 */
	public boolean navigationClick(int status, int time)  {
		if ((status & KeypadListener.STATUS_TRACKWHEEL) != KeypadListener.STATUS_TRACKWHEEL)
		{
			return selectAction();
		}

		return super.navigationClick(status, time);
	}





}
