/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/


package com.compactbyte.bibleplus.bb;

import com.compactbyte.bibleplus.reader.BiblePlusPDB;
import com.compactbyte.bibleplus.reader.BookInfo;
import com.compactbyte.bibleplus.reader.PDBDataStream;
import com.compactbyte.bibleplus.reader.Bookmark;
import net.rim.device.api.collection.ReadableList;
import net.rim.device.api.ui.component.KeywordProvider;
import net.rim.device.api.util.StringUtilities;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.Font;
import net.rim.device.api.system.Clipboard;

import java.io.IOException;
import java.util.Vector;

final class BibleModel implements ReadableList, KeywordProvider {

	Vector historylist = new Vector();
	Vector biblefilelist = new Vector();

	int getBibleFileCount() {
		return biblefilelist.size();
	}

	final static String HISTORY_FILE = "bible-history.cfg";
	final static String BIBLELIST_FILE = "bible-list.cfg";

	void clearHistory() {
		historylist.removeAllElements();
		saveHistory();
	}

	void removeBible(int i) {
		if (i<0 || i>=biblefilelist.size())
			return;
		biblefilelist.removeElementAt(i);
		saveBibleFileList();
	}

	void addBible(BibleFile b) {
		if (b==null) 
			return;
		for (int i = 0; i < biblefilelist.size(); i++) {
			BibleFile c = (BibleFile)biblefilelist.elementAt(i);
			if (c.equals(b))
				return;
		}
		biblefilelist.addElement(b);
		saveBibleFileList();
	}


	Bookmark temphistory;

	void markHistory() {
		if (bible==null)
			return;

		//ModelSpecific.log("adding bookmark 2");
		currentVerse = getVerseFromCursorPos(bibleview.getCursorPosition());
		temphistory = bible.createBookMark(currentBook, currentChapter, currentVerse);

	}

	void commitHistory() {
		if (temphistory==null) {
			return;
		}
		if (historylist.size()>0) {
			Bookmark top = (Bookmark)historylist.elementAt(historylist.size()-1);
			if (top.equals(temphistory)) {
				temphistory = null;
				return;
			}
		}
		//historylist.addElement(temphistory);
		historylist.insertElementAt(temphistory, 0);
		saveHistory();
		temphistory = null;
	}

	void rollbackHistory() {
		temphistory = null;
	}

	BibleFile getBibleFile(int i) {
		return (BibleFile)biblefilelist.elementAt(i);
	}

	BibleFile[] getBibleList() {
		BibleFile[] h = new BibleFile[biblefilelist.size()];
		biblefilelist.copyInto(h);
		return h;
	}

	Bookmark getHistory(int i) {
		return (Bookmark)historylist.elementAt(i);
	}

	Bookmark[] getHistoryList() {
		Bookmark[] h = new Bookmark[historylist.size()];
		historylist.copyInto(h);
		return h;
	}

	boolean saveBibleFileList() {
		String storepath = ModelSpecific.getConfigStoragePath();
		if (storepath==null || storepath.length()==0)
			return false;

		storepath += BIBLELIST_FILE;
		Config c = new Config();
		c.set("biblecount", biblefilelist.size());

		for (int i = 0; i < biblefilelist.size(); i++) {
			BibleFile bf = (BibleFile)biblefilelist.elementAt(i);
			c.set("bible" + i, bf.toStringRepresentation());
		}
		//ModelSpecific.log("storing to "+ storepath);
		return c.store(storepath);		
	}

	String getVersionName() {
		if (bible==null)
			return "";
		return bible.getVersionName();
	}

	boolean loadBibleFileList() {
		String storepath = ModelSpecific.getConfigStoragePath();
		if (storepath==null || storepath.length()==0)
			return false;

		storepath += BIBLELIST_FILE;

		ModelSpecific.log("storepath " + storepath);

		Config cs = new Config();
		if (!cs.load(storepath)) {
			return false;
		}
		int count = cs.getInt("biblecount", 0);

		ModelSpecific.log("count " + count);

		biblefilelist.removeAllElements();
		for (int i = 0; i < count; i++) {
			String s = cs.get("bible" + i);
			if (s==null)
				continue;
			ModelSpecific.log("sr " + s);
			BibleFile bf = BibleFile.fromStringRepresentation(s);
			ModelSpecific.log("bf " + bf);
			if (bf==null) continue;
			biblefilelist.addElement(bf);
		}
		return true;
	}



	boolean saveHistory() {
		String storepath = ModelSpecific.getConfigStoragePath();
		if (storepath==null || storepath.length()==0)
			return false;

		storepath += HISTORY_FILE;

		Config c = new Config();
		c.set("historycount", historylist.size());

		for (int i = 0; i < historylist.size(); i++) {
			Bookmark bm = (Bookmark)historylist.elementAt(i);
			c.set("history" + i, bm.toStringRepresentation());
		}
		//ModelSpecific.log("storing to "+ storepath);
		return c.store(storepath);
	}

	boolean loadHistory() {
		String storepath = ModelSpecific.getConfigStoragePath();
		if (storepath==null || storepath.length()==0)
			return false;

		storepath += HISTORY_FILE;

		Config cs = new Config();
		if (!cs.load(storepath)) {
			return false;
		}
		int count = cs.getInt("historycount", 0);
		historylist.removeAllElements();
		for (int i = 0; i < count; i++) {
			String s = cs.get("history" + i);
			if (s==null)
				continue;
			Bookmark bm = Bookmark.fromStringRepresentation(s);
			if (bm==null)
				continue;
			historylist.addElement(bm);
		}
		return true;
	}


	BookmarkManager bm = new BookmarkManager();

	BookmarkManager getBookmarkManager() {
		return bm;
	}

	/*KeywordProvider interface*/
	public String[] getKeywords(Object element)  {
		if (element instanceof BookInfo) {
			BookInfo bi = (BookInfo)element;
			return StringUtilities.stringToKeywords(bi.getFullName());
		} else {
			ModelSpecific.log("element is not instance of bookinfo");
			return null;
		}
	}


	/*readable list interface*/
	public Object getAt(int index)  {
		if (bible==null) {
			return null;
		}
		BookInfo[] b = bible.getBooksInfo();
		return b[index];
	}

	public int getAt(int index, int count, Object[] elements, int destIndex) {
		BookInfo[] b = bible.getBooksInfo();
		if (index < 0)
			throw new ArrayIndexOutOfBoundsException(index);

		for (int i = 0;  i < count; i++) {
			if (index + i>=b.length)
				return i;
			elements[destIndex + i] = b[index + i];
		}
		return count;
	}

	public int getIndex(Object element) {
		BookInfo[] b = bible.getBooksInfo();
		for (int i =0; i < b.length; i++) {
			if (element==b[i])
			    return i;
		}
		return -1;
	}

	public int  size() {
		if (bible==null) {
			return 0;
		}
		BookInfo[] b = bible.getBooksInfo();
		return b.length;
	}
	/***********************/


	String bibleSelectionDialogState;

	String getBibleSelectionDialogState() {
		return bibleSelectionDialogState;
	}

	void setBibleSelectionDialogState(String _state) {
		ModelSpecific.log("selection state " + _state);
		bibleSelectionDialogState = _state;
	}


	int currentBook;
	int currentChapter ;
	/*for goto/bookmark*/
	int currentVerse;

	int cursorPos;
	int verticalPos;

	BiblePlusPDB bible;

	BiblePlusPDB getBible() {
		return bible;
	}

	String currentChapterContent;

	String currentlyOpenPath;

	int [] versepos;

	BookInfo book;

	BookInfo getCurrentBook() {
		return book;
	}

	final static String CONFIG_FILE = "bible-plus.cfg";


	void setCursorPosition(int pos) {
		cursorPos = pos;
	}

	void setVerticalPosition(int pos) {
		verticalPos = pos;
	}

	int getVerticalPosition() {
		return verticalPos;
	}

	int getCursorPosition() {
		return cursorPos;
	}

	//Save state
	boolean saveState() {
		Config c = new Config();
		c.set("currentBook", currentBook);
		c.set("currentChapter", currentChapter);
		c.set("currentVerse", currentVerse);
		c.set("cursorPos", cursorPos);
		c.set("verticalPos", verticalPos);
		c.set("bibleSelectionDialogState", bibleSelectionDialogState);
		if (currentlyOpenPath!=null) {
			c.set("currentlyOpenPath", currentlyOpenPath);
		}
		String storepath = ModelSpecific.getConfigStoragePath();
		if (storepath==null || storepath.length()==0)
			return false;

		storepath += CONFIG_FILE;

		return c.store(storepath);
	}


	boolean loadState() {
		String storepath = ModelSpecific.getConfigStoragePath();
		if (storepath==null || storepath.length()==0)
			return false;
		storepath += CONFIG_FILE;
		Config cs = new Config();
		if (!cs.load(storepath)) {
			errorMessage  = "file not loaded";
		}
		String path = cs.get("currentlyOpenPath");
		errorMessage = "path: "+ path;
		if (path!=null) {
			if (loadBibleFile(path)) {
				currentBook = cs.getInt("currentBook", 0);
				currentChapter = cs.getInt("currentChapter", 1);
				if (currentChapter < 1 )
					currentChapter = 1;
				currentVerse = cs.getInt("currentVerse", 1);
				if (currentVerse < 1)
					currentVerse = 1;
				cursorPos = cs.getInt("cursorPos", 0);
				verticalPos = cs.getInt("verticalPos", 0);
				bibleSelectionDialogState = cs.get("bibleSelectionDialogState");
			} else {
				errorMessage = path;
				return false;
			}
			setBook(currentBook);
			loadChapter();
		} else {
			errorMessage = "path is null ";
		}
		return true;
	}

	private BibleView bibleview;

	private final static int FONT_FOR_TEXT = 0;
	private final static int FONT_FOR_NUMBER = 1;


	private Font[] fonts;

	Font[] getFonts() {
		return fonts;
	}

	private int[] offsets;
	private byte[] attributes;

	int[] getOffsets() {
		return offsets;
	}

	byte[] getAttributes() {
		return attributes;
	}

	void setBibleView(BibleView _bibleview) {
		bibleview = _bibleview;
		fonts = new Font[2];
		fonts[FONT_FOR_TEXT] = bibleview.getFont();
		fonts[FONT_FOR_NUMBER] = fonts[FONT_FOR_TEXT].derive(Font.BOLD);
	}

	BibleModel() {
		currentBook = 0;
		currentChapter = 1;
		currentVerse = 1;
	}


	String errorMessage;

	String getBookName() {
		if (book==null)
			return "";
		return book.getFullName();
	}

	int getCurrentChapter() {
		return currentChapter;
	}

	boolean isBibleLoaded() {
		return bible!=null;
	}

	int getCurrentVerse() {
		return currentVerse;
	}

	boolean gotoVerse(String location) {
		if (bible==null)
			return false;

		/*just number, to jump to a verse*/
		int v = -1;

		try {
			v = Integer.parseInt(location);
		} catch (NumberFormatException e) {

		}

		if (v>0) {
			int verseCount = book.getVerseCount(currentChapter);
			if (v<=verseCount) {
				currentVerse = v;
				cursorPos = getVerseCursorPos(currentVerse);
				return true;
			}
		}


		int[] r = bible.parseQuery(location);
		if (r==null)
			return false;

		BookInfo oldbookinfo = book;

		try {
			book = bible.getBook(r[0]);
			book.openBook();
		} catch (IOException e) {
			book = oldbookinfo;
			return false;
		}
		currentBook = r[0];
		currentChapter = r[1];
		currentVerse = r[2];
		loadChapter();
		return true;
	}

	void updateCurrentVerse() {
		currentVerse = getVerseFromCursorPos(bibleview.getCursorPosition());
	}

	boolean loadBibleFile(String path) {
		BiblePlusPDB _oldbible = bible;
		BookInfo _oldbook = book;

		try {
			errorMessage = "opening file";

			//ModelSpecific.log("opening file " + path);

			PDBDataStream fcs = ModelSpecific.openDataStream(path);
			ModelSpecific.log("opening " + path);
			errorMessage = "constructing bible";
			bible = new BiblePlusPDB(fcs);
			errorMessage = "loading version info";
			if (!bible.loadVersionInfo()) {
				bible.close();
				bible = _oldbible;
				book = _oldbook;
				return false;
			}
			errorMessage = "loading  word index";
			bible.loadWordIndex();
			errorMessage = "opening book " + currentBook;

			//ModelSpecific.log("cc " + currentChapter);

			BookInfo nb[] = bible.getBooksInfo();


			//ModelSpecific.log(">1< verse " + currentVerse);

			/*todo: for better handling: switch to
			  corresponding booknumber (if exists)*/
			if (currentBook>=bible.getBookCount()) {
				currentBook = 0;
				currentChapter = 1;
				currentVerse = 1;
				cursorPos = 0;
				verticalPos = 0;
			}

			//ModelSpecific.log(">2< verse " + currentVerse);
			book = bible.getBook(currentBook);

			if (_oldbible!=null) {
				errorMessage = "closing old bible";
				_oldbible.close();
				errorMessage = "done closing old bible";
			}

			currentlyOpenPath = path;
			verticalPos = -1;

			//ModelSpecific.log("cc2 " + currentChapter);
			//ModelSpecific.log(">3< verse " + currentVerse);
			return true;
		} catch (Exception e) {
			ModelSpecific.log("1error " + e + ":err:"+errorMessage);
			try {
				bible.close();
			} catch (Exception e2) {
				ModelSpecific.log("2error2 " + e + ":err:"+errorMessage);
			}
			errorMessage += e;
			bible = _oldbible;
			book = _oldbook;
			return false;
		}
	}


	boolean loadBible(String path) {
		if (currentlyOpenPath!=null) {
			if (path.equalsIgnoreCase(currentlyOpenPath)) {
				return true;
			}
		}
		return loadBibleFile(path);
	}


	boolean setBook(int bookidx) {
		if (bible==null)
			return false;
		try  {
			BookInfo tmpbook = bible.getBook(bookidx);
			tmpbook.openBook();
			book = tmpbook;
			currentBook = bookidx;
			currentChapter = 1;
			currentVerse = 1;
			cursorPos = 0;
			verticalPos = 0;
			return true;
		} catch (Exception e) {
			return false;
		}
	}

	void setChapter(int chapter) {
		currentChapter = chapter;
	}

	boolean nextChapter() {
		if (book==null)
			return false;
		if (currentChapter==book.getChapterCount()) {
			return false;
		}
		currentChapter++;
		loadChapter();
		return true;
	}

	boolean prevChapter() {
		if (book==null)
			return false;
		if (currentChapter==1) {
			return false;
		}
		currentChapter--;
		loadChapter();
		return true;
	}


	int getVerseCursorPos(int v) {
		ModelSpecific.log("V = " + v);
		if (versepos==null) {
			ModelSpecific.log("verse pos is NULL");
			return 0;
		}
		if (v<1 || v>versepos.length) {
			ModelSpecific.log("verse index not valid");
			return 0;
		}

		// StringBuffer sb = new StringBuffer();
		// for (int i =0; i < versepos.length; i++) {
		// 	sb.append(versepos[i]).append(" ");
		// }
		// ModelSpecific.log(sb.toString());


		return versepos[v-1];
	}

	int getVerseFromCursorPos(int cpos) {
		if (versepos==null || book==null)
			return 1;
		for (int i = 1; i < versepos.length; i++) {
			if (cpos<versepos[i])
				return i;
		}
		return book.getVerseCount(currentChapter);
	}


	int lastLoadedBook = 0;
	int lastLoadedChapter = 0;
	String lastLoadedPath = null;
	StringBuffer chaptersb = new StringBuffer();


	void loadChapter() {

		long _start = System.currentTimeMillis();

		if (currentBook==lastLoadedBook && currentChapter==lastLoadedChapter) {
			if (lastLoadedPath!=null) {
				if (lastLoadedPath.equals(currentlyOpenPath)) {
					return ;
				}
			}
		}


		try {
			book.openBook();
		} catch (IOException e) {
			return ;
		}

		int verseCount = book.getVerseCount(currentChapter);

		chaptersb.setLength(0);

		versepos = new int[verseCount];

		offsets = new int[verseCount*2+1];

		attributes = new byte[verseCount*2];


		int off = 0;
		int att = 0;

		offsets[off++] = chaptersb.length();

		for (int v = 1; v <= verseCount; v++) {

			String s = book.getVerse(currentChapter, v);
			versepos[v-1] = chaptersb.length();
			chaptersb.append(v).append(" ");
			attributes[att++] = (byte)FONT_FOR_NUMBER;
			offsets[off++] = chaptersb.length();
			chaptersb.append(s);
			chaptersb.append("\n");
			attributes[att++] = (byte)FONT_FOR_TEXT;
			offsets[off++] = chaptersb.length();
		}

		offsets[offsets.length-1] = chaptersb.length();

		currentChapterContent = chaptersb.toString();
		lastLoadedBook = currentBook;
		lastLoadedChapter = currentChapter;
		lastLoadedPath = currentlyOpenPath;
		ModelSpecific.benchmark("Loading chapter ", _start);
	}

	String getChapterContent() {
		return currentChapterContent;
	}

	boolean addBookmark() {
		//ModelSpecific.log("adding bookmark 1");
		if (bible==null)
			return false;
		//ModelSpecific.log("adding bookmark 2");
		currentVerse = getVerseFromCursorPos(bibleview.getCursorPosition());
		//ModelSpecific.log("create bokomark " + currentBook + " " + currentChapter +  " " + currentVerse);
		Bookmark newbm = bible.createBookMark(currentBook, currentChapter, currentVerse);
		//ModelSpecific.log("adding bookmark 3"+newbm);
		return bm.addBookmark(newbm);
	}

	boolean gotoBookmark(Bookmark bookmark) {
		if (bookmark==null)
			return false;

		int booknum = bookmark.getBookNumber();

		int p = bible.findBookNumber(booknum);

		if (p<0)
			return false;

		int chapter = bookmark.getChapter();
		int verse  = bookmark.getVerse();

		BookInfo oldbookinfo = book;

		if (p!=book.getBookPosition()) {

			try {
				book = bible.getBook(p);
				book.openBook();
			} catch (IOException e) {
				book = oldbookinfo;
				return false;
			}
		}
		currentBook = p;
		currentChapter = chapter;
		currentVerse = verse;
		//ModelSpecific.log("current cursor " + bibleview.getCursorPosition());
		//ModelSpecific.log(">> chapter " + currentChapter  + " verse " + currentVerse);
		verticalPos = -1;
		loadChapter();
		//ModelSpecific.log("2>> chapter " + currentChapter  + " verse " + currentVerse);
		cursorPos = getVerseCursorPos(currentVerse);
		//ModelSpecific.log("cursor pos = " + cursorPos);
		return true;
	}

	boolean copyVerse() {
		if (bible==null) 
			return false;
		updateCurrentVerse(); //update currentverse from cursor position
		String s = book.getVerse(currentChapter, currentVerse);
		String loc  = " - " + getBookName() + " " + getCurrentChapter() + 
			":" + getCurrentVerse();
		Clipboard clip = Clipboard.getClipboard();
		String toCopy = s + loc;
		clip.put(toCopy);
		return true;
	}


}
