/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.bb;


import net.rim.blackberry.api.browser.Browser;
import net.rim.blackberry.api.browser.BrowserSession;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.container.VerticalFieldManager;
import net.rim.device.api.i18n.*;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.system.Display;
import net.rim.device.api.ui.Font;
import java.io.*;

import com.compactbyte.bibleplus.reader.BiblePlusPDB;
import com.compactbyte.bibleplus.reader.BookInfo;
import com.compactbyte.bibleplus.reader.PDBDataStream;



final class BiblePlusScreen extends MainScreen implements BiblePlusAppResource, FieldChangeListener
{

	LabelField title ;
	BibleView rf;
	WaitMessageDialog wait_popup;

	private static ResourceBundle _resources = ResourceBundle.getBundle(BUNDLE_ID, BUNDLE_NAME);

	public BiblePlusScreen()   {
		model = new BibleModel();

		title = new LabelField(_resources.getString(APPLICATION_TITLE) , LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
		setTitle(title);

		_searchScreen = new SearchScreen(model);
		_searchResultScreen = new SearchResultScreen(model, _searchScreen);

		rf = new BibleView(_resources.getString(NO_BIBLE_LOADED), Field.FOCUSABLE|Field.READONLY|Field.USE_ALL_WIDTH|
			Field.USE_ALL_HEIGHT);

		model.setBibleView(rf);

		rf.setChangeListener(this);

		add(rf);
		rf.setFocus();

		rf.setText(_resources.getString(LOADING_BIBLE));

		wait_popup = new WaitMessageDialog(_resources.getString(PROCESSING));


		new Thread(new Runnable(){
					public void run() {
						model.loadState();
						model.loadHistory();
						model.loadBibleFileList();
						updateLater();
					}
			}).start();

	}

	void updateLater() {
		UiApplication.getUiApplication().invokeLater(new Runnable() {
				public void run() 	{
					if (model.isBibleLoaded()) {
						refreshView(true);
						if (model.getVerticalPosition()>=0) {
							rf.getManager().setVerticalScroll(model.getVerticalPosition());
						}						
					} else {
						rf.setText(_resources.getString(NO_BIBLE_LOADED));
					}
				}
		});
	}

	public void fieldChanged(Field field, int context) {
		if (field==rf) {
			if (context==BibleView.NEXT_CHAPTER) {
				if (model!=null) {
					if (model.nextChapter()) {
						refreshView(false);
						rf.setCursorPosition(0);
					}
				}
			}
			if (context==BibleView.PREV_CHAPTER) {
				if (model!=null) {
					if (model.prevChapter()) {
						refreshView(false);
						rf.setCursorPosition(rf.getTextLength()-1);
					}
				}
			}
		}
	}


	BibleModel model;

	int lastOffsetsLength = -1;
	String lastContent = null;

	void refreshView(boolean setCursor) {
		try {

			if (!model.isBibleLoaded()) {
					return;
			}

			model.loadChapter();			

			int[] offsets = model.getOffsets();
			byte[] attributes = model.getAttributes();
			Font[] fonts = model.getFonts();

			String content = model.getChapterContent();

			if (offsets!=null && attributes!=null && fonts!=null) {
				if (offsets.length!=lastOffsetsLength ||  
				  content!=lastContent) {
					long _start = System.currentTimeMillis();
					rf.setText(content, offsets, attributes, fonts);
					lastOffsetsLength = offsets.length;
					lastContent = content;
					ModelSpecific.benchmark("Rendering chapter ", _start);
				}
			} else {
				rf.setText(content);
			}
			title.setText(model.getBookName() + " " + model.getCurrentChapter());
			if (setCursor) {
				//ModelSpecific.log("set pos " + model.getCursorPosition());
				rf.setCursorPosition(model.getCursorPosition());
			}
		} catch (Exception e) {
			Dialog.inform("Error:"+e);
		}
	}

	private MenuItem _gotoVerseItem = new MenuItem(_resources, GOTO_VERSE, 20, 10) {
			public void run() {
				if (!model.isBibleLoaded()) {
					return;
				}
				GotoVerseScreen gotoVerseScreen = new GotoVerseScreen();
				UiApplication.getUiApplication().pushModalScreen(gotoVerseScreen);
				String q = gotoVerseScreen.getText();
				if (q.trim().equals(""))
					return;
				model.markHistory();
				if (model.gotoVerse(q)) {
					refreshView(true);
					model.commitHistory();
				} else {
					model.rollbackHistory();
					Dialog.inform(_resources.getString(VERSE_NOT_FOUND));
				}
			}
		};

	private MenuItem _selectBookItem = new MenuItem(_resources, SELECT_BOOK, 10, 10) {
			public void run() {
				if (!model.isBibleLoaded()) {
					return;
				}
				BookListScreen bookListScreen = new BookListScreen(model);
				UiApplication.getUiApplication().pushModalScreen(bookListScreen);
				if (bookListScreen.getSelectedBook()==-1)
					return;
				int book = bookListScreen.getSelectedBook();
				int chapter = bookListScreen.getSelectedChapter();
				ModelSpecific.log("Select book " + book + " chapter " + chapter);
				if (model.setBook(book)) {
					model.setChapter(chapter);
					model.setCursorPosition(0);
					refreshView(true);
				}
			}
		};


	SearchScreen _searchScreen;
	SearchResultScreen _searchResultScreen;


	private void showSearchScreen() {
		if (!model.isBibleLoaded()) {
			return;
		}

		UiApplication.getUiApplication().pushModalScreen(_searchScreen);
		if (_searchScreen.getSearchResultList()!=null) {
			UiApplication.getUiApplication().pushModalScreen(_searchResultScreen);
			if (_searchResultScreen.isSearchResultSelected()) {
				model.loadChapter();
				model.setCursorPosition(model.getVerseCursorPos(model.getCurrentVerse()));
				refreshView(true);
					}
		} else {
			if (_searchScreen.isSearchPerformed()) {
				Dialog.inform(_resources.getString(SEARCH_RESULT_NOT_FOUND));
			}
		}
	}

	private MenuItem _searchResultItem = new MenuItem(_resources, SEARCH_RESULT, 60, 10) {
			public void run() {
				if (!model.isBibleLoaded()) {
					return;
				}

				if (_searchScreen.getSearchResultList()!=null) {
					UiApplication.getUiApplication().pushModalScreen(_searchResultScreen);
					if (_searchResultScreen.isSearchResultSelected()) {
						model.loadChapter();
						model.setCursorPosition(model.getVerseCursorPos(model.getCurrentVerse()));
						refreshView(true);
						return;
					}
				} else {
					showSearchScreen();
				}
			}
		};


	private MenuItem _searchItem = new MenuItem(_resources, SEARCH, 50, 10) {
			public void run() {
				if (!model.isBibleLoaded()) {
					return;
				}

				showSearchScreen();
			}
		};


	private MenuItem _aboutItem = new MenuItem(_resources, ABOUT, 100, 10) {
			public void run() {
				AboutScreen about_screen = new AboutScreen(model);
				UiApplication.getUiApplication().pushModalScreen(about_screen);
			}
		};

	private MenuItem _selectBibleItem = new MenuItem(_resources, SELECT_BIBLE, 30, 10) {
			public void run() {
				if (model.getBibleFileCount()==0) {
					_addBibleItem.run();
					return;
				}
				model.markHistory();
				model.updateCurrentVerse(); //update currentverse from cursor position

				BibleSelectionScreen selection = new BibleSelectionScreen(model);
				UiApplication.getUiApplication().pushModalScreen(selection);
				if (selection.isBibleLoaded()) {
					model.loadChapter();
					model.setCursorPosition(model.getVerseCursorPos(model.getCurrentVerse()));
					refreshView(true);
					if (!model.saveState()) {
						Dialog.inform("failed saving state");
					}
					model.commitHistory();
				} else {
					model.rollbackHistory();
				}
				
			}
		};

	private MenuItem _addBibleItem = new MenuItem(_resources, ADD_BIBLE, 85, 10) {
			public void run() {

				model.markHistory();
				model.updateCurrentVerse(); //update currentverse from cursor position
				AddBibleScreen add_bible_screen = new AddBibleScreen(model);
				UiApplication.getUiApplication().pushModalScreen(add_bible_screen);
				if (add_bible_screen.isBibleLoaded()) {
					//ModelSpecific.log(">> verse " + model.getCurrentVerse());
					model.loadChapter();
					model.setCursorPosition(model.getVerseCursorPos(model.getCurrentVerse()));
					refreshView(true);
					if (!model.saveState()) {
						Dialog.inform("failed saving state");
					}
					model.commitHistory();
				} else {
					model.rollbackHistory();
				}

			}
		};

	/*
	private MenuItem _selectEncodingItem = new MenuItem(_resources, SELECT_ENCODING, 20, 10) {
			public void run() {

			}
		};
	*/

	private MenuItem _downloadBibleItem = new MenuItem(_resources, DOWNLOAD_BIBLE, 90, 10) {
			public void run() {
				String url = _resources.getString(DOWNLOAD_URL);
				BrowserSession bSession = Browser.getDefaultSession();
				bSession.displayPage(url);			
			}
		};

	private MenuItem _copyVerseItem = new MenuItem(_resources, COPY_VERSE, 50, 10) {
			public void run() {
				if (!model.isBibleLoaded()) {
					return;
				}

				if (model.copyVerse()) {
					Dialog.inform(_resources.getString(VERSE_COPIED));
				}
			}
		};


	private MenuItem _showHistoryItem = new MenuItem(_resources, SHOW_HISTORY, 60, 10) {
			public void run() {
				if (!model.isBibleLoaded()) {
					return;
				}

				HistoryScreen history_selection = new HistoryScreen(model);
				UiApplication.getUiApplication().pushModalScreen(history_selection);
				if (history_selection.isHistorySelected()) {
					model.loadChapter();
					model.setCursorPosition(model.getVerseCursorPos(model.getCurrentVerse()));
					refreshView(true);
				}

			}
		};

	private MenuItem _addBookmarkItem = new MenuItem(_resources, ADD_BOOKMARK, 70, 10) {
			public void run() {
				if (!model.isBibleLoaded()) {
					return;
				}

				ModelSpecific.log("adding bookmark");
				if (model.addBookmark()) {
					ModelSpecific.log("done adding bookmark");
					Dialog.inform(_resources.getString(BOOKMARK_ADDED));
				}
			}
		};




	private MenuItem _showBookmarksItem = new MenuItem(_resources, SHOW_BOOKMARKS, 80, 10) {
			public void run() {
				if (!model.isBibleLoaded()) {
					return;
				}

				model.markHistory();
				BookmarksScreen bookmark_selection = new BookmarksScreen(model);
				UiApplication.getUiApplication().pushModalScreen(bookmark_selection);
				if (bookmark_selection.isBookmarkSelected()) {
					model.loadChapter();
					model.setCursorPosition(model.getVerseCursorPos(model.getCurrentVerse()));
					refreshView(true);
					model.commitHistory();
				} else {
					model.rollbackHistory();
				}
			}
		};

	/*
	private MenuItem _showOptions = new MenuItem(_resources, SHOW_OPTIONS, 20, 10) {
			public void run() {
				
			}
		};
	*/


	protected void makeMenu(Menu menu, int instance) {
		super.makeMenu(menu, instance);
		menu.add(_selectBookItem);
		menu.add(_gotoVerseItem);
		menu.add(_selectBibleItem);
		menu.add(_copyVerseItem);
		menu.add(_addBibleItem);
		menu.add(_searchItem);
		menu.add(_searchResultItem);
		/*menu.add(_selectEncodingItem);*/
		menu.add(_showHistoryItem);
		menu.add(_addBookmarkItem);
		menu.add(_showBookmarksItem);
		/*menu.add(_showOptions);*/
		menu.add(_downloadBibleItem);
		menu.add(_aboutItem);
	}


	public boolean onClose()   {
		if  (Dialog.ask(Dialog.D_YES_NO, _resources.getString(CONFIRM_EXIT),
				Dialog.NO)==Dialog.YES) {
			close();
			return true;
		} 
		return false;
	}


	/**
	 * Display a dialog box to the user with "Goodbye!!!" when the application
	 * is closed.
	 *
	 * @see net.rim.device.api.ui.Screen#close()
	 */
	public void close()   {
		//todo: save state here
		model.setCursorPosition(rf.getCursorPosition());
		model.setVerticalPosition(rf.getManager().getVerticalScroll());
		ModelSpecific.log("vscroll " +rf.getManager().getVerticalScroll());
		model.saveState();
		System.exit(0);
		super.close();
	}
}
