/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.bb;

import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.component.TextField;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.component.ObjectListField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.i18n.ResourceBundle;
import net.rim.device.api.system.Characters;
import javax.microedition.io.file.*;
import javax.microedition.io.Connector;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Stack;
import java.io.*;
import com.compactbyte.bibleplus.reader.Bookmark;       
import javax.microedition.io.file.FileSystemRegistry;

import com.compactbyte.bibleplus.reader.Util;
import com.compactbyte.bibleplus.reader.MyStringTokenizer;
import net.rim.device.api.system.KeypadListener;

final class BibleSelectionScreen extends MainScreen implements BiblePlusAppResource
{
	LabelField title ;
	ObjectListField bibleList;
	
	private static ResourceBundle _resources = ResourceBundle.getBundle(BUNDLE_ID, BUNDLE_NAME);

	WaitMessageDialog waitdialog;

	BibleModel model;

	public BibleSelectionScreen(BibleModel _model)   {

		waitdialog = new WaitMessageDialog(_resources.getString(LOADING_BIBLE));

		model = _model;
		title = new LabelField(_resources.getString(SELECT_BIBLE_SCREEN_TITLE), 
				       LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
		setTitle(title);
		bibleList = new ObjectListField();
		bibleList.set(model.getBibleList());
		add(bibleList);
	}


	boolean bibleLoaded = false;

	boolean isBibleLoaded() {
		return bibleLoaded;
	}


	void closeScreenLater() {
		UiApplication.getUiApplication().invokeLater(new Runnable() {
				public void run() {
					UiApplication uiapp = UiApplication.getUiApplication();
					uiapp.popScreen(uiapp.getActiveScreen());
				}
			});
	}

	void informFailedLater() {
		UiApplication.getUiApplication().invokeLater(new Runnable() {
				public void run() {
					Dialog.inform(_resources.getString(FAILED_LOADING_BIBLE));					
				}
			});
	}

	String pathToLoad;

	private void loadBibleFileInDifferentThread() {
		ModelSpecific.log("starting thread");
		new Thread(new Runnable(){
				public void run() {
					waitdialog.show();
					if (model.loadBible(pathToLoad)) {
						bibleLoaded = true;
						model.loadChapter();
						waitdialog.close();
						closeScreenLater();
					} else {
						waitdialog.close();
						informFailedLater();
					}
				}
			}).start();
	}


	private boolean selectBible() {
		int i  = bibleList.getSelectedIndex();
		if (i<0) return false;
		BibleFile b = model.getBibleFile(i);
		pathToLoad = b.getLocation();
		loadBibleFileInDifferentThread();
		return true;
	}

	/**
	 * Overrides default.  Enter key will take action 
	 * 
	 * @see net.rim.device.api.ui.Screen#keyChar(char,int,int)
	 * 
	 */
	public boolean keyChar(char c, int status, int time)    {
		if (c==Characters.ENTER) {
			return selectBible(); 
		}
		return super.keyChar(c, status, time);
		
	}


	/**
	 * Overrides default implementation.  Performs the select action if the 
	 * trackwheel was clicked; otherwise, the default action occurs.
	 * 
	 * @see net.rim.device.api.ui.Screen#navigationClick(int,int)
	 */
	public boolean navigationClick(int status, int time)  {
		if ((status & KeypadListener.STATUS_TRACKWHEEL) != KeypadListener.STATUS_TRACKWHEEL) {
			return selectBible();
		}
		
		return super.navigationClick(status, time);
	}

	private void removeBible() {
		int i  = bibleList.getSelectedIndex();		
		model.removeBible(i);
		bibleList.set(model.getBibleList());		
	}

	private MenuItem _selectBibleItem = new MenuItem(_resources, SELECT_BIBLE_ACTION, 20, 10) {
			public void run() {
				selectBible();
			}
		};

	private MenuItem _removeBibleItem = new MenuItem(_resources, REMOVE_BIBLE_ACTION, 20, 20) {
			public void run() {
				removeBible();
				bibleList.set(model.getBibleList());
			}
		};

	
	protected void makeMenu(Menu menu, int instance) {
		super.makeMenu(menu, instance);
		menu.add(_selectBibleItem);
		menu.add(_removeBibleItem);

	}


}
