/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.bb;

import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.component.TextField;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.i18n.ResourceBundle;
import net.rim.device.api.ui.XYRect;
import net.rim.device.api.system.Characters;
import net.rim.device.api.ui.component.KeywordFilterField;

import java.io.IOException;

import com.compactbyte.bibleplus.reader.BiblePlusPDB;
import com.compactbyte.bibleplus.reader.BookInfo;

final class BookListScreen extends MainScreen implements BiblePlusAppResource, FieldChangeListener
{
	KeywordFilterField bookInputField;

	private static ResourceBundle _resources = ResourceBundle.getBundle(BUNDLE_ID, BUNDLE_NAME);

	BibleModel model;

	public BookListScreen(BibleModel _model)   {
		bookInputField = new KeywordFilterField();
		model = _model;
		bookInputField.setSourceList(model, model);		
		setTitle(bookInputField.getKeywordField());
		bookInputField.setLabel(_resources.getString(SELECT_BOOK_QUERY));
		add(bookInputField);
	}

	public void fieldChanged(Field field, int context) {
		
	}

	int selectedBook = -1;
	int selectedChapter = 0;

	int getSelectedBook() {
		return selectedBook;
	}

	int getSelectedChapter() {
		return selectedChapter;
	}

	private void bookSelected() {
		BookInfo book = (BookInfo)bookInputField.getSelectedElement(); 

		try {
			book.openBook();
		} catch (IOException e) {
			return;
		}

		ChapterListScreen ch = new ChapterListScreen(model, book.getFullName(), book.getChapterCount());
		UiApplication.getUiApplication().pushModalScreen(ch);
		if (ch.getSelectedChapter()==0) {
			return;
		}
		
		selectedChapter = ch.getSelectedChapter();

		selectedBook = book.getBookPosition();
		close();

	}

	/**
	 * Handles a trackball click.
	 * @see net.rim.device.api.ui.Screen#invokeAction(int)
	 */
	public boolean invokeAction(int action)   {        
		switch(action) {
		case ACTION_INVOKE: // Trackball click.
			bookSelected();
			return true; // We've consumed the event.
		}    
		return  super.invokeAction(action);
	}

	/**
	 * Intercepts the ENTER key and displays info screen. 
	 * @see net.rim.device.api.ui.Screen#keyChar(char,int,int)
	 */   
	protected boolean keyChar(char key, int status, int time) {
		if (key == Characters.ENTER) {
			bookSelected();
			return true; // We've consumed the event.    
		}             
		return super.keyChar(key, status, time);
	}


}
