/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.bb;

import com.compactbyte.bibleplus.reader.BiblePlusPDB;
import com.compactbyte.bibleplus.reader.BookInfo;       
import com.compactbyte.bibleplus.reader.PDBDataStream;       
import com.compactbyte.bibleplus.reader.Bookmark;       
import net.rim.device.api.collection.ReadableList;
import net.rim.device.api.ui.component.KeywordProvider;
import net.rim.device.api.util.StringUtilities;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.Font;

import java.io.IOException;
import java.util.Vector;

final class BookmarkManager implements ReadableList, KeywordProvider {

	final static String BOOKMARKS_FILE = "bible-bookmark.cfg";

	Vector bookmarks = new Vector();

	/*KeywordProvider interface*/
	public String[] getKeywords(Object element)  {
		if (element instanceof Bookmark) {
			Bookmark b = (Bookmark)element;
			return StringUtilities.stringToKeywords(b.toString());
		} else {
			ModelSpecific.log("element is not instance of bookmark");
			return null;
		}
	}


	/*readable list interface*/
	public Object getAt(int index)  {
		return bookmarks.elementAt(index);
	}

	public int getAt(int index, int count, Object[] elements, int destIndex) {
		if (index < 0) 
			throw new ArrayIndexOutOfBoundsException(index);
		
		for (int i = 0;  i < count; i++) {
			if (index + i>=bookmarks.size())
				return i;
			elements[destIndex + i] = bookmarks.elementAt(index + i);
		}
		return count;
	}

	public int getIndex(Object element) {
		for (int i =0; i < bookmarks.size(); i++) {
			if (element==bookmarks.elementAt(i))
			    return i;
		}
		return -1;
	}

	public int  size() {
		return bookmarks.size();
	}
	/***********************/

	boolean saveBookmarks() {
		String storepath = ModelSpecific.getConfigStoragePath();
		if (storepath==null || storepath.length()==0) 
			return false;
		
		storepath += BOOKMARKS_FILE;
		
		Config c = new Config();
		c.set("bookmarkcount", bookmarks.size());
		
		for (int i = 0; i < bookmarks.size(); i++) {
			Bookmark bm = (Bookmark)bookmarks.elementAt(i);
			ModelSpecific.log("storing "+ bm);
			ModelSpecific.log("storing "+ bm.getLocation());
			ModelSpecific.log("storing "+ bm.getVersion());

			c.set("bookmark" + i, bm.toStringRepresentation());			
		}
		ModelSpecific.log("storing to "+ storepath);

		return c.store(storepath);
	}

	boolean loadBookmarks() {
		String storepath = ModelSpecific.getConfigStoragePath();
		if (storepath==null || storepath.length()==0) 
			return false;
		
		storepath += BOOKMARKS_FILE;

		Config cs = new Config();
		if (!cs.load(storepath)) {
			return false;
		}
		int count = cs.getInt("bookmarkcount", 0);
		bookmarks.removeAllElements();
		for (int i = 0; i < count; i++) {
			String s = cs.get("bookmark" + i);
			if (s==null) 
				continue;
			Bookmark bm = Bookmark.fromStringRepresentation(s);
			bookmarks.addElement(bm);
		}
		return true;
	}


	boolean addBookmark(Bookmark newbm) {

		ModelSpecific.log("adding bookmark 4");
		for (int i = 0; i < bookmarks.size(); i++) {
			Bookmark current = (Bookmark) bookmarks.elementAt(i);
			if (newbm.equals(current))
				return false;
		}
		ModelSpecific.log("adding bookmark 5"+newbm);
		//bookmarks.addElement(newbm);
		bookmarks.insertElementAt(newbm, 0);
		ModelSpecific.log("adding bookmark 6");
		saveBookmarks();
		ModelSpecific.log("adding bookmark 7");
		return true;

	}

	boolean removeBookmark(Bookmark bm) {
		if (bookmarks.size()==0) {
			return false;
		}
		int sz = bookmarks.size();
		bookmarks.removeElement(bm);
		saveBookmarks();
		return sz!=bookmarks.size();
	}

}
