/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.bb;

import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.component.TextField;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.component.ObjectListField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.i18n.ResourceBundle;
import net.rim.device.api.system.Characters;
import javax.microedition.io.file.*;
import javax.microedition.io.Connector;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Stack;
import java.io.*;
import javax.microedition.io.file.FileSystemRegistry;
import net.rim.device.api.ui.component.KeywordFilterField;

import com.compactbyte.bibleplus.reader.Util;
import com.compactbyte.bibleplus.reader.Bookmark;
import com.compactbyte.bibleplus.reader.MyStringTokenizer;

final class BookmarksScreen extends MainScreen implements BiblePlusAppResource
{
	LabelField title ;
	ObjectListField bookmarkList;

	KeywordFilterField filterField;

	private static ResourceBundle _resources = ResourceBundle.getBundle(BUNDLE_ID, BUNDLE_NAME);

	BibleModel model;


	private MenuItem _deleteBookmarkItem = new MenuItem(_resources, DELETE_BOOKMARK, 20, 10) {
			public void run() {
				removeSelectedBookmark();
			}
		};


	protected void makeMenu(Menu menu, int instance) {
		super.makeMenu(menu, instance);
		menu.add(_deleteBookmarkItem);
	}


	boolean bookmarkSelected = false;

	boolean isBookmarkSelected() {
		return bookmarkSelected;
	}
	
	public BookmarksScreen(BibleModel _model)   {
		filterField = new KeywordFilterField();
		_model.getBookmarkManager().loadBookmarks();
		model = _model;
		filterField.setSourceList(model.getBookmarkManager(), model.getBookmarkManager());		
		setTitle(filterField.getKeywordField());		
		filterField.setLabel(_resources.getString(BOOKMARK_FILTER_QUERY));
		add(filterField);		
	}

	private void removeSelectedBookmark() {
		Bookmark bookmark = (Bookmark)filterField.getSelectedElement(); 

		if (bookmark==null) {
			ModelSpecific.log("bookmark is null");
			return;
		}

		if (Dialog.ask(Dialog.D_DELETE, _resources.getString(CONFIRM_BOOKMARK_DELETE),
			       Dialog.NO)==Dialog.DELETE) {
			model.getBookmarkManager().removeBookmark(bookmark);
			filterField.updateList();
		} 
	}



	private void bookmarkSelected() {
		Bookmark bookmark = (Bookmark)filterField.getSelectedElement(); 

		bookmarkSelected = true;

		model.gotoBookmark(bookmark);

		close();

	}


	/**
	 * Handles a trackball click.
	 * @see net.rim.device.api.ui.Screen#invokeAction(int)
	 */
	public boolean invokeAction(int action)   {        
		switch(action) {
		case ACTION_INVOKE: // Trackball click.
			bookmarkSelected();
			return true; // We've consumed the event.
		}    
		return  super.invokeAction(action);
	}

	/**
	 * Intercepts the ENTER key and displays info screen. 
	 * @see net.rim.device.api.ui.Screen#keyChar(char,int,int)
	 */   
	protected boolean keyChar(char key, int status, int time) {
		if (key == Characters.ENTER) {
			bookmarkSelected();
			return true; // We've consumed the event.    
		}             
		return super.keyChar(key, status, time);
	}
	

}
