/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.bb;

import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.component.TextField;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.i18n.ResourceBundle;
import net.rim.device.api.ui.XYRect;
import net.rim.device.api.system.Characters;
import net.rim.device.api.ui.component.KeywordFilterField;

import java.io.IOException;

import com.compactbyte.bibleplus.reader.BiblePlusPDB;
import com.compactbyte.bibleplus.reader.BookInfo;
import net.rim.device.api.collection.ReadableList;
import net.rim.device.api.ui.component.KeywordProvider;
import net.rim.device.api.util.StringUtilities;


final class ChapterListScreen extends MainScreen implements BiblePlusAppResource, FieldChangeListener
{
	KeywordFilterField chapterInputField;
	StringList chapterlist;
	int selectedChapter = 0;

	int getSelectedChapter() {
		return selectedChapter;
	}

	private void chapterSelected() {
		String cha = (String)chapterInputField.getSelectedElement(); 
		int c = chapterlist.getIndex(cha)+1;
		if (c<1)
			c = 1;
		selectedChapter = c;
		close();
	}

	/**
	 * Handles a trackball click.
	 * @see net.rim.device.api.ui.Screen#invokeAction(int)
	 */
	public boolean invokeAction(int action)   {        
		switch(action) {
		case ACTION_INVOKE: // Trackball click.
			chapterSelected();
			return true; // We've consumed the event.
		}    
		return  super.invokeAction(action);
	}

	/**
	 * Intercepts the ENTER key and displays info screen. 
	 * @see net.rim.device.api.ui.Screen#keyChar(char,int,int)
	 */   
	protected boolean keyChar(char key, int status, int time) {
		if (key == Characters.ENTER) {
			chapterSelected();
			return true; // We've consumed the event.    
		}             
		return super.keyChar(key, status, time);
	}


	private static ResourceBundle _resources = ResourceBundle.getBundle(BUNDLE_ID, BUNDLE_NAME);

	public ChapterListScreen(BibleModel model, String bookname, int count)   {
		chapterInputField = new KeywordFilterField();		
		chapterlist = new StringList(_resources.getString(CHAPTER_PREFIX), count);
		chapterInputField.setSourceList(chapterlist, chapterlist);		
		setTitle(chapterInputField.getKeywordField());
		chapterInputField.setLabel(bookname + ": ");
		add(chapterInputField);
	}

	
	public void fieldChanged(Field field, int context) {
		
	}


	final static class StringList implements ReadableList, KeywordProvider {

		String[] list;

		StringList(String prefix, int count) {
			list = new String[count];
			for (int i = 0 ; i < count; i++) {
				list[i] = prefix + (i+1);
			}
		}

		/*KeywordProvider interface*/
		public String[] getKeywords(Object element)  {			
			if (element instanceof String) {
				String s = (String)element;
				return StringUtilities.stringToKeywords(s);
			} 
			return null;
		}


		/*readable list interface*/
		public Object getAt(int index)  {
			return list[index];
		}

		public int getAt(int index, int count, Object[] elements, int destIndex) {
			if (index < 0) 
				throw new ArrayIndexOutOfBoundsException(index);
			
			for (int i = 0;  i < count; i++) {
				if (index + i>=list.length)
					return i;
				elements[destIndex + i] = list[index + i];
			}
			return count;
		}

		public int getIndex(Object element) {
			for (int i =0; i < list.length; i++) {
				if (element==list[i])
					return i;
			}
			return -1;
		}

		public int  size() {
			return list.length;
		}
	}	
}
