/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.bb;

import java.util.Hashtable;
import java.util.Enumeration;
import java.io.IOException;
import java.io.DataOutputStream;
import java.io.DataInputStream;
import javax.microedition.io.file.*;
import javax.microedition.io.Connector;

/*
 * Simple portable data storage scheme 
 * Not using sqlite (only available since 5.0)
 */

final class Config {

	private Hashtable hashtable;

	public Config() {
		hashtable = new Hashtable();
	}

	void set(String prop, String val) {
		if (val==null)  {
			Object o = hashtable.get(prop);
			if (o!=null) {
				hashtable.remove(prop);
			}
			return;
		}			
		hashtable.put(prop, val);
	}

	void set(String prop, int val) {
		String s = Integer.toString(val);
		hashtable.put(prop, s);
		//ModelSpecific.log("set " + prop + " to " + s);
	}

	String get(String prop) {
		return (String)hashtable.get(prop);
	}

	int getInt(String prop, int def) {
		String s = (String)hashtable.get(prop);
		if (s==null)
			return def;
		try {
			return Integer.parseInt(s);
		} catch (NumberFormatException e) {
			return def;
		}
	}


	boolean store(String path) {
		try {
			FileConnection fconn = (FileConnection)Connector.open(path);
			if (!fconn.exists())
				fconn.create();  // create the file if it doesn't exist
			DataOutputStream dos = fconn.openDataOutputStream();

			dos.writeInt(hashtable.size());
			//ModelSpecific.log("have " + hashtable.size() + " entries");
			for (Enumeration k = hashtable.keys(); k.hasMoreElements(); ) {
				String key = (String)k.nextElement();
				dos.writeUTF(key);
				String value = (String)hashtable.get(key);
				dos.writeUTF(value);
				//ModelSpecific.log("write " + key + " " + value);
			}
			dos.flush();
			dos.close();
			fconn.close();
			return true;
		} catch (IOException e) {
			return false;
		}
	}

	boolean load(String path) {
		try {
			//ModelSpecific.log("opening " + path);
			FileConnection fconn = (FileConnection)Connector.open(path);
			if (!fconn.exists()) {
				//ModelSpecific.log("file doesn't exist");
				fconn.close();
				return false;
			}

			DataInputStream dis = fconn.openDataInputStream();
			Hashtable newht = new Hashtable();

			int n = dis.readInt();
			//ModelSpecific.log("have " + n + " entries");
			for (int k = 0; k<n; k++ ) {
				String key = dis.readUTF();
				String value = dis.readUTF();
				//ModelSpecific.log("read " + key + " " + value);
				newht.put(key, value);
			}
			hashtable = newht;
			fconn.close();
			return true;
		} catch (IOException e) {
			ModelSpecific.log("error" + e);
			return false;
		}

	}

}