/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.bb;

import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.component.TextField;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.component.ObjectListField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.i18n.ResourceBundle;
import net.rim.device.api.system.Characters;
import javax.microedition.io.file.*;
import javax.microedition.io.Connector;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Stack;
import java.io.*;
import com.compactbyte.bibleplus.reader.Bookmark;       
import javax.microedition.io.file.FileSystemRegistry;

import com.compactbyte.bibleplus.reader.Util;
import com.compactbyte.bibleplus.reader.MyStringTokenizer;
import net.rim.device.api.system.KeypadListener;

final class HistoryScreen extends MainScreen implements BiblePlusAppResource
{
	LabelField title ;
	ObjectListField historyList;
	
	private static ResourceBundle _resources = ResourceBundle.getBundle(BUNDLE_ID, BUNDLE_NAME);


	private void clearHistory() {
		model.clearHistory();
		historyList.set(model.getHistoryList());
	}

	BibleModel model;

	public HistoryScreen(BibleModel _model)   {
		model = _model;
		title = new LabelField(_resources.getString(APPLICATION_TITLE) , LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
		setTitle(title);
		historyList = new ObjectListField();
		historyList.set(model.getHistoryList());
		add(historyList);
	}


	boolean historySelected = false;

	boolean isHistorySelected() {
		return historySelected;
	}

	private boolean selectHistory() {
		int i  = historyList.getSelectedIndex();
		if (i<0) return false;
		Bookmark b = model.getHistory(i);
		model.gotoBookmark(b);
		historySelected = true;
		close();		
		return true;
	}

	/**
	 * Overrides default.  Enter key will take action 
	 * 
	 * @see net.rim.device.api.ui.Screen#keyChar(char,int,int)
	 * 
	 */
	public boolean keyChar(char c, int status, int time)    {
		if (c==Characters.ENTER) {
			return selectHistory(); 
		}
		return super.keyChar(c, status, time);
		
	}


	/**
	 * Overrides default implementation.  Performs the select action if the 
	 * trackwheel was clicked; otherwise, the default action occurs.
	 * 
	 * @see net.rim.device.api.ui.Screen#navigationClick(int,int)
	 */
	public boolean navigationClick(int status, int time)  {
		if ((status & KeypadListener.STATUS_TRACKWHEEL) != KeypadListener.STATUS_TRACKWHEEL) {
			return selectHistory();
		}
		
		return super.navigationClick(status, time);
	}


	private MenuItem _clearHistoryItem = new MenuItem(_resources, CLEAR_HISTORY, 20, 10) {
			public void run() {
				clearHistory();
				historyList.set(model.getHistoryList());
			}
		};


	private MenuItem _selectHistoryItem = new MenuItem(_resources, SELECT_HISTORY, 20, 10) {
			public void run() {
				selectHistory();
			}
		};

	
	protected void makeMenu(Menu menu, int instance) {
		super.makeMenu(menu, instance);
		menu.add(_selectHistoryItem);
		menu.add(_clearHistoryItem);
	}


}
