/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.bb;

import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.GaugeField;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.component.TextField;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.ObjectListField;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.i18n.*;
import net.rim.device.api.ui.container.DialogFieldManager;
import net.rim.device.api.ui.container.PopupScreen;
import net.rim.device.api.ui.component.ObjectChoiceField;
import net.rim.device.api.ui.component.RadioButtonGroup;
import net.rim.device.api.ui.component.RadioButtonField;

import java.io.*;
import java.util.Vector;

import com.compactbyte.bibleplus.reader.BiblePlusPDB;
import com.compactbyte.bibleplus.reader.BookInfo;
import com.compactbyte.bibleplus.reader.BookRange;
import com.compactbyte.bibleplus.reader.BookRanges;
import com.compactbyte.bibleplus.reader.Bookmark;
import com.compactbyte.bibleplus.reader.SearchProgressListener;
import com.compactbyte.bibleplus.reader.MyStringTokenizer;
import net.rim.device.api.system.Characters;
import net.rim.device.api.system.KeypadListener;

final class SearchResultScreen extends MainScreen implements BiblePlusAppResource
{
	LabelField title ;

	private static ResourceBundle _resources = ResourceBundle.getBundle(BUNDLE_ID, BUNDLE_NAME);

	BibleModel model;
	SearchScreen searchScreen;

	ObjectListField resultList;

	public SearchResultScreen(BibleModel _model, SearchScreen _searchScreen)   {
		model = _model;
		searchScreen = _searchScreen;
		title = new LabelField(_resources.getString(SEARCH_RESULT) , LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
		setTitle(title);
		resultList = new ObjectListField();
		add(resultList);
	}

	public void onDisplay() {
		super.onDisplay();
		searchResultSelected = false;
		resultList.set(searchScreen.getSearchResultList());		
	}

	boolean searchResultSelected = false;

	boolean isSearchResultSelected() {
		return searchResultSelected;
	}

	private boolean selectSearchResult() {
		int i  = resultList.getSelectedIndex();
		if (i<0) return false;
		Bookmark b = searchScreen.getResult(i);
		model.gotoBookmark(b);
		searchResultSelected = true;
		close();		
		return true;
	}
       
	
	/**
	 * Overrides default.  Enter key will take action 
	 * 
	 * @see net.rim.device.api.ui.Screen#keyChar(char,int,int)
	 * 
	 */
	public boolean keyChar(char c, int status, int time)    {
		if (c==Characters.ENTER) {
			return selectSearchResult(); 
		}
		return super.keyChar(c, status, time);
		
	}
	
	public boolean navigationClick(int status, int time)  {
		if ((status & KeypadListener.STATUS_TRACKWHEEL) != KeypadListener.STATUS_TRACKWHEEL) {
			return selectSearchResult();
		}
		
		return super.navigationClick(status, time);
	}


	private MenuItem _selectSearchResultItem= new MenuItem(_resources, SELECT_SEARCH_RESULT, 20, 10) {
			public void run() {
				selectSearchResult();
			}
		};

	
	protected void makeMenu(Menu menu, int instance) {
		super.makeMenu(menu, instance);
		menu.add(_selectSearchResultItem);
	}

	
}
