/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.bb;

import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.GaugeField;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.component.TextField;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.i18n.*;
import net.rim.device.api.ui.container.DialogFieldManager;
import net.rim.device.api.ui.container.PopupScreen;
import net.rim.device.api.ui.component.ObjectChoiceField;
import net.rim.device.api.ui.component.RadioButtonGroup;
import net.rim.device.api.ui.component.RadioButtonField;

import java.io.*;
import java.util.Vector;

import com.compactbyte.bibleplus.reader.BiblePlusPDB;
import com.compactbyte.bibleplus.reader.BookInfo;
import com.compactbyte.bibleplus.reader.BookRange;
import com.compactbyte.bibleplus.reader.Bookmark;
import com.compactbyte.bibleplus.reader.BookRanges;
import com.compactbyte.bibleplus.reader.SearchProgressListener;
import com.compactbyte.bibleplus.reader.MyStringTokenizer;
import net.rim.device.api.system.KeypadListener;
import net.rim.device.api.system.Characters;

final class SearchScreen extends MainScreen implements BiblePlusAppResource, FieldChangeListener
{
	LabelField title ;
	EditField tf;
	ButtonField searchButton;
	RadioButtonGroup rangeChoice;

	final int SEARCH_LIMIT = 200;

	private static ResourceBundle _resources = ResourceBundle.getBundle(BUNDLE_ID, BUNDLE_NAME);
	BookRange[] ranges;
	RadioButtonField[] radiobuttons;
	ProgressBarDialog progressbar;

	BibleModel model;
	Vector searchResult;

	Bookmark [] searchResultList;			

	boolean searchPerformed = false;

	boolean isSearchPerformed() {
		return searchPerformed;
	}

	public void onDisplay() {
		super.onDisplay();
		searchPerformed = false;
	}


	Bookmark[] getSearchResultList() {
		return searchResultList;
	}
	
	Bookmark getResult(int i) {
		return searchResultList[i];
	}

	public SearchScreen(BibleModel _model)   {

		model = _model;

		title = new LabelField(_resources.getString(SEARCH) , LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
		setTitle(title);
		tf = new EditField(_resources.getString(SEARCH_LABEL), "", 140, EditField.NO_NEWLINE);
		add(tf);

		ranges  = new BookRange[4];
		radiobuttons = new RadioButtonField[4];
		rangeChoice = new RadioButtonGroup();

		ranges[0] = BookRanges.getRange(BookRanges.ALL_BOOKS_FROM_THIS_POINT);
		ranges[0].setName(_resources.getString(ALL_BOOKS_FROM_THIS_POINT));
		ranges[1] = BookRanges.getRange(BookRanges.ALL_BOOKS);
		ranges[1].setName(_resources.getString(ALL_BOOKS));
		ranges[2] = BookRanges.getRange(BookRanges.OLD_TESTAMENT);
		ranges[2].setName(_resources.getString(OLD_TESTAMENT));
		ranges[3] = BookRanges.getRange(BookRanges.NEW_TESTAMENT);
		ranges[3].setName(_resources.getString(NEW_TESTAMENT));
		
		for (int i =0; i < 4; i++) {
			radiobuttons[i] = new RadioButtonField(ranges[i].toString(), rangeChoice, i==0);
			add(radiobuttons[i]);
		}
		
		searchButton = new ButtonField(_resources.getString(SEARCH_BUTTON), ButtonField.CONSUME_CLICK);
		add(searchButton);

		progressbar = new ProgressBarDialog("Searching");

		searchButton.setChangeListener(this);

	}

	void closeLater() {
		UiApplication.getUiApplication().invokeLater(new Runnable() {
				public void run() {								
					close();
				}
			});
	}

	protected boolean onSavePrompt() {
		return true;
	}


	public void fieldChanged(Field field, int context) {
		if (field==searchButton) {
			
			model.getBible().setSearchProgressListener(progressbar);
			searchPerformed = true;
		       
			new Thread(new Runnable(){
					public void run() {
						MyStringTokenizer st = new MyStringTokenizer(tf.getText(), " ,.:/?");
						String[] q = st.getTokens();

						int sel = rangeChoice.getSelectedIndex();
						if (sel==0) {
							BookInfo currentbook = model.getCurrentBook();
							int global_index = model.getBible().findGlobalBookNumberIndex(currentbook.getBookNumber());
							ranges[sel].setStart(
									     global_index, 
									     model.getCurrentChapter(), 
									     model.getCurrentVerse());
						}						

						//ModelSpecific.log("query: " + tf.getText() + " " + q.length);
						searchResultList = null;
						searchResult = model.getBible().search(q,
										   ranges[sel],
										   true,
										   true, SEARCH_LIMIT);
						if (searchResult!=null && searchResult.size()>0) {
							BiblePlusPDB bible = model.getBible();
							int count= searchResult.size()/3;
							searchResultList = new Bookmark[count];
							for (int i =0; i < count; i++) {
								int book = ((Integer)searchResult.elementAt(i*3)).intValue();
								int chapter = ((Integer)searchResult.elementAt(i*3+1)).intValue();
								int verse = ((Integer)searchResult.elementAt(i*3+2)).intValue();
								Bookmark bm = bible.createBookMark(book, chapter, verse);
								searchResultList[i] = bm;
							}
						}
						closeLater();
					}
				}).start();
		}
	}



	/**
	 * Creates a popup dialog box containing a gauge field , displays the 
	 * progress as the list is populated because it might take a while.
	 */    
	class ProgressBarDialog implements SearchProgressListener, FieldChangeListener
	{
		private DialogFieldManager _manager;
		private PopupScreen _popupScreen;
		private GaugeField _gaugeField;
		private LabelField _lbfield;
		ButtonField _cancelButton;
        
		private int _max;
		private boolean _canceled = false;

		public void fieldChanged(Field field, int context) {
			if (field==_cancelButton) {
				_canceled = true;
			}
		}
        
		/**
		 * Constructor
		 * 
		 * @param title      Text to display on _popupScreen.
		 * @param max        Maximum value of the range _gaugeField can display.
		 */
		public ProgressBarDialog(String title )	{
			_manager = new DialogFieldManager();
			_popupScreen = new PopupScreen(_manager);
			_lbfield = new LabelField(title , Field.USE_ALL_WIDTH);                         
			_cancelButton = new ButtonField(_resources.getString(CANCEL_SEARCH_BUTTON), ButtonField.CONSUME_CLICK);
			_cancelButton.setChangeListener(this);
			_manager.addCustomField(_lbfield);
		}

		
                
		public void searchFinished() 	{
			// Remove _popupScreen from the stack.
			UiApplication.getUiApplication().invokeLater(new Runnable() {
					public void run() 	{								
						UiApplication.getUiApplication().popScreen(_popupScreen) ; 
					}
				});        
			}

		public void searchStarted(int maxstep) {
			_max = maxstep;
			//ModelSpecific.log("maxstep "+maxstep);
			UiApplication.getUiApplication().invokeLater(new Runnable() {
					public void run() {			
						if (_gaugeField==null) {
							_gaugeField = new GaugeField( null , 0 , _max , 0 , GaugeField.PERCENT);
							_manager.addCustomField(_gaugeField);            
							_manager.addCustomField(_cancelButton);            
							_canceled = false;
						} else {
							_gaugeField.setValue(0);
						}
						UiApplication.getUiApplication().pushScreen(_popupScreen);  
					}
			});

		}

		public void notify(int currentstep, int maxstep, 
				   int currentbook, int found) {
			if (_gaugeField!=null) {
				_gaugeField.setValue(currentstep);
			}
		}

		public boolean searchCanceled() {
			return _canceled;
		}

	}

}
