/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.reader;

import java.io.IOException;
import java.util.Stack;

public class BookInfo {

	/*book number*/
	int bookNum;
	/*index record*/
	int bookIndex;
	int totalBookRec;
	int bookPosition;
	String simpleName;
	String complexName;
	BiblePlusPDB bible;

	public int getBookNumber() {
		return bookNum;
	}

	public String getFullName() {
		return complexName;
	}

	public String getShortName() {
		return simpleName;
	}

	int totalChapters;

	boolean bookOpened = false;


	public final int bookTextType = 0xFFFF;
	public final int chapTextType = 0xFFFE;
	public final int descTextType = 0xFFFD;
	public final int versTextType = 0xFFFC;


	public int RECORD_SIZE = 4096;

	byte index_data[];
	byte data[];
	int totalVersesAcc[];
	int totalChapterCharsAcc[];
	int totalVerseCharsAcc[];

	byte[] getData() {
		return data;
	}

	/**
	 * Get number of chapters in this book
	 * @return chapter count
	 */
	public int getChapterCount() {
		return totalChapters;
	}

	StringBuffer[] sb;

	/*
	int getBookIndex() {
		return bookIndex;
	}
	*/
	BookInfo(BiblePlusPDB bible) {
		this.bible = bible;
		sb = new StringBuffer[4];

		for (int i =0; i < 4; i++) {
			sb[i] = new StringBuffer();
		}

	}

	public String toString() {
		return complexName;
	}

	public String toStringInfo() {
		StringBuffer sb = new StringBuffer();
		sb.append("Book Num: ").append(bookNum).append("\n");
		sb.append("Book Index: ").append(bookIndex).append("\n");
		sb.append("Book rec: ").append(totalBookRec).append("\n");
		sb.append("Name: ").append(simpleName).append("\n");
		sb.append("Full Name: ").append(complexName).append("\n");
		return sb.toString();
	}

	int getVerseStart(int chapter, int verse) {
		//System.out.println("vstart = " + (chapter==1?0:totalVersesAcc[chapter - 2]));
		int verseAcc = (chapter==1?0:totalVersesAcc[chapter - 2]) + verse;
		//System.out.println("verseacc = " + verseAcc);
		int verseStart = chapter==0?0:totalChapterCharsAcc[chapter-1];
		//System.out.println("versestart1: " + verseStart);

		if (verse > 1){
			verseStart += verseAcc==1?0:totalVerseCharsAcc[verseAcc - 2];
		}
		//System.out.println("versestart2: " + verseStart);

		return verseStart;
	}

	public int getVerseCount(int chapter) {
		int v1 = totalVersesAcc[chapter-1];
		int v2 = chapter==1?0:totalVersesAcc[chapter-2];
		return v1-v2;
	}


	int vlen(int index) {
		//System.out.println("vlen = " + index);
		int v1 = totalVerseCharsAcc[index];
		int v2 = index==0?0:totalVerseCharsAcc[index-1];
		int diff = v1 - v2;
		if (diff<0) return 0;
		return diff;
	}


	int getVerseLength(int chapter, int verse) {
		int verseAcc = (chapter==1?0:totalVersesAcc[chapter - 2]) + verse;
		int verseLength;
		if (verse > 1){
			verseLength = vlen(verseAcc-1);
		} else {
			verseLength = verseAcc==0?0:totalVerseCharsAcc[verseAcc-1];
		}
		return verseLength;
	}

	int shiftLookup[] = { 0, 3, 2, 1 };
	int verseShiftLookup[] = { 10, 4, 6, 8 };

	StringBuffer[] getVerseByteShifted(int chapter, int verse) {
		int verseStart = getVerseStart(chapter, verse);
		int verseLength = getVerseLength(chapter, verse);

		for (int i =0; i < 4; i++) {
			sb[i].setLength(0);
		}

		System.out.println("Start " + verseStart + " length " + verseLength);

		int decShift = 0;
		int decValueBuffer[] = new int[3];

		int compStart;
		compStart = verseStart * 7 / 4;
		decShift = shiftLookup[verseStart * 7 % 4];

		int idx = compStart;

		switch (decShift) {
		case 1:
			decValueBuffer[1] = data[idx++];
			break;
		case 2:
		case 3:
			decValueBuffer[2] = data[idx++];
			break;
		default:
			;
		}

		int sbpos = 0;

		for (int i = 0; i < verseLength; i++) {

			switch (decShift) {
			case 0:
				decValueBuffer[0] = data[idx++] & 0xff;
				decValueBuffer[1] = data[idx++] & 0xff;
				decValueBuffer[2] = 0;
				break;
			case 1:
				decValueBuffer[0] = decValueBuffer[1];
				decValueBuffer[1] = data[idx++] & 0xff;
				decValueBuffer[2] = data[idx++] & 0xff;
				break;
			case 2:
				decValueBuffer[0] = decValueBuffer[2];
				decValueBuffer[1] = data[idx++] & 0xff;
				decValueBuffer[2] = data[idx++] & 0xff;
				break;
			case 3:
				decValueBuffer[0] = decValueBuffer[2];
				decValueBuffer[1] = data[idx++]  & 0xff;
				decValueBuffer[2] = 0;
				break;
			default:
				;
			}

			int value = decValueBuffer[0] << 16 | decValueBuffer[1] << 8 |
				decValueBuffer[2];
			value = value >> verseShiftLookup[decShift];
			value = value & 0x3FFF;
			decShift++;

			if (decShift == 4) {
				decShift = 0;
			}

			if (value > 0x3FF0) {
				value |= 0xC000;
			}

			int decWordNum = value;
			int pos = bible.getWordPos(decWordNum);
			int wordIndex = bible.getWordIndex(pos, decWordNum);
			int[] r = bible.getRepeat(pos, decWordNum);
			if (r!=null) {
				for (int j = 0; j < r.length; j++) {
					if (r[j] > 0x3FF0) {
						r[j] |= 0xC000;
					}

					if (r[j]==bookTextType || r[j]==chapTextType ||
					    r[j]==descTextType || r[j]==versTextType){
						sbpos = r[j] - versTextType;
						continue;
					}

					String word = bible.getWord(r[j]);

					if (sb[sbpos].length()>0) {
						if (word.length()==1) {
							switch (word.charAt(0)) {
							case '.':
							case ',':
							case '?':
							case '!':
							case ';':
							case ':':
							case '-':
								break;
							default:
								sb[sbpos].append(bible.getSepChar());
							}

						} else {
							sb[sbpos].append(bible.getSepChar());
						}
					}

					sb[sbpos].append(word);
				}
			}
		}
		return sb;
	}

	public StringBuffer[] getCompleteVerse(int chapter, int verse) {

		if (!bookOpened) {
			return null;
		}

		if (chapter <0 || chapter>getChapterCount()) {
			return null;
		}

		if (verse<0 || verse>getVerseCount(chapter)) {
			return null;
		}

		if (bible.isByteShifted()) {
			return getVerseByteShifted(chapter, verse);
		}

		for (int i =0; i < 4; i++) {
			sb[i].setLength(0);
		}



		int sbpos = 0; //verse


		int verseStart = getVerseStart(chapter, verse);
		//System.out.println("versestart " + verseStart);

		int verseLength = getVerseLength(chapter, verse);

		//System.out.println("Start " + verseStart + " length " + verseLength);

		int compStart = verseStart * 2;

		int idx = compStart;

		for (int i = 0; i < verseLength; i++) {
			int decWordNum = (data[idx] & 0xff) * 256 + (data[idx + 1] & 0xff);
			idx += 2;
			//System.out.println("decWordNum " + decWordNum);
			int pos = bible.getWordPos(decWordNum);
			//System.out.println("wordpos " + pos);
			//System.out.println("wordlength " + bible.getWordLength(pos));
			int wordIndex = bible.getWordIndex(pos, decWordNum);
			//System.out.println("wordindex " + wordIndex);
			int[] r = bible.getRepeat(pos, decWordNum);
			if (r!=null) {
				for (int j = 0; j < r.length; j++) {
					//if (r[j]<0) break;
					//System.out.print(r[j]+" ");

					if (r[j]==bookTextType || r[j]==chapTextType ||
					    r[j]==descTextType || r[j]==versTextType){
						sbpos = r[j] - versTextType;
						continue;
					}


					String word = bible.getWord(r[j]);

					if (sb[sbpos].length()>0) {
						if (word.length()==1) {
							switch (word.charAt(0)) {
							case '.':
							case ',':
							case '?':
							case '!':
							case ';':
							case ':':
							case '-':
								break;
							default:
								sb[sbpos].append(bible.getSepChar());
							}

						} else {
							sb[sbpos].append(bible.getSepChar());
						}
					}
					sb[sbpos].append(word);
				}
			}
		}

		return sb;

	}


	public String getVerse(int chapter, int verse) {
		StringBuffer[] sb = getCompleteVerse(chapter, verse);
		if (sb==null) return null;
		return sb[0].toString();
	}


	public void openBook() throws IOException {

		if (bookOpened)
			return;

		if (!bible.supportRandomAccess()) {
			//if we don't have random access
			//must make sure all previous books have been opened
			//the correct way to do it is to load all previous books in main thread, and make progress bar
			//this is to prevent error
			for (int i = 0; i < bookPosition; i++) {
				BookInfo bi = bible.getBook(i);
				bi.openBook();
			}
		}

		PDBRecord r = bible.getPDBAccess().readRecord(bookIndex);
		index_data = r.getData();

		//Util.dumpBytes(index_data);

		PDBRecord [] records = new PDBRecord[totalBookRec];
		int total_len = 0;
		for (int i = 0;  i<totalBookRec; i++) {
			records[i] = bible.getPDBAccess().readRecord(bookIndex+i+1);
			byte []d = records[i].getData();
			//System.out.println("d = "+ d.length);
			total_len += d.length;
		}
		data = new byte[total_len];
		for (int i = 0;  i<totalBookRec; i++) {
			byte []d = records[i].getData();
			System.arraycopy(d, 0, data, i*RECORD_SIZE, d.length);
			bible.getPDBAccess().removeFromCache(bookIndex+i+1);
		}

		totalChapters = Util.readShort(index_data, 0);

		System.out.println("total chapters: " + totalChapters);

		totalVersesAcc = new int[totalChapters];

		int offs = 2;

		for (int i = 0; i < totalChapters; i++) {
			totalVersesAcc[i] = Util.readShort(index_data, offs);
			//System.out.println("totalVersesAcc " + totalVersesAcc[i]);
			offs += 2;
		}

		totalChapterCharsAcc = new int[totalChapters];

		for (int i = 0; i < totalChapters; i++) {
			totalChapterCharsAcc[i] = Util.readInt(index_data, offs);
			offs += 4;
		}


		totalVerseCharsAcc = new int[(index_data.length-offs)/2];


		for (int i = 0; offs < index_data.length; i++) {
			totalVerseCharsAcc[i] = Util.readShort(index_data, offs);
			offs +=2;
		}

		System.out.println("total length " + total_len  + " total chapter " + totalChapters);

		bookOpened = true;

	}

	public int getBookPosition() {
		return bookPosition;
	}

	public static BookInfo createFromData(BiblePlusPDB bible, byte data[], int offset, int book_pos) {
		BookInfo bi = new BookInfo(bible);
		int idx = 0;
		bi.bookPosition = book_pos;
		bi.bookNum = Util.readShort(data, offset + idx);
		idx += 2;
		bi.bookIndex = Util.readShort(data, offset + idx);
		idx += 2;
		bi.totalBookRec = Util.readShort(data, offset + idx);
		if (bi.bookIndex + bi.totalBookRec > bible.getHeader().getRecordCount()) {
			return null;
		}

		idx += 2;
		bi.simpleName = Util.readStringTrimZero(data, offset + idx, 8, bible.getEncoding());
		idx += 8;
		bi.complexName = Util.readStringTrimZero(data, offset + idx, 32, bible.getEncoding());
		return bi;
	}

	void close() {
		index_data = null;
		data = null;
		totalVersesAcc = null;
		totalChapterCharsAcc = null;
		totalVerseCharsAcc = null;
	}

}
