/*
    Bible Plus A Bible Reader for Blackberry
    Copyright (C) 2010 Yohanes Nugroho

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Yohanes Nugroho (yohanes@gmail.com)
*/

package com.compactbyte.bibleplus.reader;

import java.io.DataInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;

class PDBHeader {       
	/* 78 bytes total */
	//char	 name[ dmDBNameLength ]; /*0*/
	byte[] _name;
	int attributes;  //32
	int version; //34
	int create_time; //36
	int modify_time; //40
	int backup_time; //44
	int modification_number; //48
	int app_info_id; //52
	int sort_info_id; //56
	String type_str; //4 char (offset 60)
	byte type[];
	String creator_str; //4 char (offset 64)
	byte creator[];
	int id_seed; // 68
	int next_record_list; //72
	int num_records; //76-78

	byte[] headerdata;


	public String getCreator() {
		return creator_str;
	}

	public String getType() {
		return type_str;
	}

	public int getRecordCount() {
		return num_records;
	}
	
	public PDBHeader(byte[] headerdata) {
		this.headerdata = headerdata;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("Name: ").append(getName("UTF-8")).append("\n");
		sb.append("type_str: ").append(type_str).append("\n");
		sb.append("creator_str: ").append(creator_str).append("\n");
		sb.append("num records: ").append(num_records).append("\n");
		return sb.toString();
	}

	int readInt(DataInputStream dis) throws IOException {
		return dis.readInt();
	}

	int readShort(DataInputStream dis) throws IOException {
		return dis.readShort();
	}

	String getName(String encoding) {
		return Util.readStringTrimZero(_name, 0, 32, encoding);
	}

	public void load() throws IOException {
		ByteArrayInputStream bis = new ByteArrayInputStream(headerdata);
		DataInputStream dis = new DataInputStream(bis);
		_name = new byte[32];
		dis.read(_name);
		attributes = dis.readShort();
		version = dis.readShort();
		create_time = readInt(dis);
		modify_time = readInt(dis);
		backup_time = readInt(dis);
		modification_number = readInt(dis);
		app_info_id = readInt(dis);
		sort_info_id = readInt(dis);
		type = new byte[4];
		dis.read(type, 0, 4);
		type_str = new String(type);		
		creator = new byte[4];		
		dis.read(creator, 0, 4);		
		creator_str = new String(creator);
		id_seed = readInt(dis);
		next_record_list = readInt(dis);
		num_records = readShort(dis);
	}

}
